<?php 
/**
 * Předpokladem je PHP verze 5.1.2 a vyšší s moduly mhash, mcrypt.
 *
 * Obsahuje pomocne funkcionality pro komunikaci s platebnim rozh-
 * ranim GoPay.
 */
class GopayHelper{

	/**
	 * URL platebni brany pro zakladni integraci
	 *
	 * @return URL
	 */
	function baseIntegrationURL(){
		return 'https://testgw.gopay.cz/zaplatit-jednoducha-integrace';		
	}

	/**
	 * URL platebni brany pro uplnou integraci
	 *
	 * @return URL
	 */
	function fullIntegrationURL(){
		return 'https://testgw.gopay.cz/zaplatit-plna-integrace';		
	}

	/**
	 * URL WS
	 *
	 * @return URL - wsdl
	 */
	function ws(){
		$wsUrl = trim(Configuration::get('GOPAY_WS_URL')); // take Prestashop configuration
		return $wsUrl;
	}		

	/**
	 * Sestaveni retezce pro podpis platebniho prikazu
	 *
	 * @param long $goId
	 * @param string $productName
	 * @param long $totalPriceInCents
	 * @param string $variableSymbol
	 * @param string $failedURL
	 * @param string $successURL
	 * @param string $key
	 * @return retezec pro podpis 
	 */
  	function concatPaymentCommand(
  		$goId,
  		$productName, 
  		$totalPriceInCents, 
  		$variableSymbol,
  		$failedURL,
  		$successURL, 
  		$key){

        return $goId."|".$productName."|".$totalPriceInCents."|".$variableSymbol."|".$failedURL."|".$successURL."|".$key; 
  	}

  	/**
  	 * Sestaveni retezce pro podpis vysledku vytvoreni platby
  	 *
  	 * @param long $goId
  	 * @param string $productName
  	 * @param long $totalPriceInCents
  	 * @param string $variableSymbol
  	 * @param string $result
  	 * @param string $sessionState
  	 * @param string $key
  	 * @return retezec pro podpis
  	 */
  	function concatPaymentResult(
  		$goId,
  		$productName, 
  		$totalPriceInCents, 
  		$variableSymbol,
  		$result,
  		$sessionState,
  		$key){

        return $goId."|".$productName."|".$totalPriceInCents."|".$variableSymbol."|".$result."|".$sessionState."|".$key; 
  	}

  	  	/**
  	 * Sestaveni retezce pro podpis vysledku stav platby
  	 *
  	 * @param long $goId
  	 * @param string $productName
  	 * @param long $totalPriceInCents
  	 * @param string $variableSymbol
  	 * @param string $result
  	 * @param string $sessionState
  	 * @param string $paymentChannel
  	 * @param string $key
  	 * @return retezec pro podpis
  	 */
  	function concatPaymentStatus(
  		$goId,
  		$productName, 
  		$totalPriceInCents, 
  		$variableSymbol,
  		$result,
  		$sessionState,
  		$paymentChannel,
  		$key){

        return $goId."|".$productName."|".$totalPriceInCents."|".$variableSymbol."|".$result."|".$sessionState."|".$paymentChannel."|".$key; 
  	}
  	
  	
  	/**
  	 * Sestaveni retezce pro podpis sessionInfo
  	 *
  	 * @param long $goId
  	 * @param long $paymentSessionId
  	 * @param string $key
  	 * @return retezec pro podpis
  	 */
  	function concatPaymentSession(
  		$goId,
  	 	$paymentSessionId,  	 	 
  	 	$key){
        return $goId."|".$paymentSessionId."|".$key; 
  	}

  	/**
  	 * Sestaveni retezce pro podpis popisu platby (paymentIdentity)
  	 *
  	 * @param long $goId
  	 * @param long $paymentSessionId
  	 * @param string $variableSymbol
  	 * @param string $key
  	 * @return retezec pro podpis
  	 */
  	function concatPaymentIdentity(
  		$goId,
  	 	$paymentSessionId,
  	 	$variableSymbol, 
  	 	$key){
        return $goId."|".$paymentSessionId."|".$variableSymbol."|".$key; 
  	}

  	/**
  	 * Sifrovani dat 3DES
  	 *
  	 * @param string $data
  	 * @param string $key
  	 * @return sifrovany obsah v HEX forme
  	 */
  	function encrypt($data, $key){
  		$td = mcrypt_module_open (MCRYPT_3DES, '', MCRYPT_MODE_ECB, '');
        $mcrypt_iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
        mcrypt_generic_init ($td, substr($key, 0, mcrypt_enc_get_key_size($td)), $mcrypt_iv);
        $encrypted_data = mcrypt_generic ($td, $data);
        mcrypt_generic_deinit ($td);
        mcrypt_module_close ($td);

        return bin2hex($encrypted_data);
  	}

  	/**
  	 * desifrovani
  	 *
  	 * @param string $data
  	 * @param string $key
  	 * @return desifrovany retezec
  	 */
  	function decrypt($data, $key){
  		$td = mcrypt_module_open (MCRYPT_3DES, '', MCRYPT_MODE_ECB, '');
        $mcrypt_iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
        mcrypt_generic_init ($td, substr($key, 0, mcrypt_enc_get_key_size($td)), $mcrypt_iv);

        $decrypted_data = mdecrypt_generic($td, GopayHelper::convert($data));
        mcrypt_generic_deinit ($td);
        mcrypt_module_close ($td);

		return Trim($decrypted_data);

  	}  	

  	/**
  	 * hash SHA1 dat
  	 *
  	 * @param string $data
  	 * @return otisk dat SHA1
  	 */
  	function hash($data){
  		if(function_exists("sha1")==true){
  			$hash = sha1($data, true);

  		} else {
  			$hash = mhash(MHASH_SHA1,$data);
  		}

  		return bin2hex($hash);   		
  	}

  	/**
  	 * konverze z HEX -> string
  	 *
  	 * @param string $hexString
  	 * @return konverze z HEX -> string
  	 */
  	function convert($hexString){

  		$hexLenght = strlen($hexString);
  		// only hex numbers is allowed                
  		if ($hexLenght % 2 != 0 || preg_match("/[^\da-fA-F]/",$hexString)) return FALSE;
  		$binString = "";
  		for ($x = 1; $x <= $hexLenght/2; $x++)                
  		{                        
  			$binString .= chr(hexdec(substr($hexString,2 * $x - 2,2)));

  		}

  		return $binString;

  	}

	/**
	 * Kontrola vysledku vytvoreni platby proti
	 * internim udajum objednavky
	 * 
	 * - verifikace podpisu
	 * 
	 *
	 * @param mixed $payment_result - vysledek volani createPayment
	 * @param string $session_state - ocekavany stav paymentSession (WAITING, PAYMENT_DONE)
	 * @param long $goId - identifikace eshopu
	 * @param string $variableSymbol - identifikace akt. objednavky
	 * @param long $priceInCents - cena objednavky v centech
	 * @param string $productName - nazev zbozi
	 * @param string $secret - klic urceny k podepisovani komunikace
	 * 
	 * @return true
	 * @return false
	 */
  	function checkPaymentResult(
  				$payment_result,
  	 			$session_state,
  	 			$goId,
  	 			$variableSymbol,  	 			
  	 			$priceInCents,
  	 			$productName,
  	 			$secret	 			
  	 			){

		$valid = true;
		
		$valid = GopayHelper::checkPaymentResultCommon(
  				$payment_result,
  	 			$session_state,
  	 			$goId,
  	 			$variableSymbol,  	 			
  	 			$priceInCents,
  	 			$productName,
  	 			$secret	 					
		);

		if( $valid ){

			$hashedSignature=GopayHelper::hash(
					GopayHelper::concatPaymentResult(
						$payment_result->eshopGoId,
						$payment_result->productName,						
						$payment_result->totalPrice,
						$payment_result->variableSymbol,
						$payment_result->result,
						$payment_result->sessionState,
						$secret)
			);			

			$decryptedHash = GopayHelper::decrypt($payment_result->encryptedSignature,$secret);

			if($decryptedHash != $hashedSignature){
				$valid = false;
//				echo "PS invalid signature <br>";
			}
		}

		return $valid;
	}
	
	/**
	 * Kontrola stavu platby proti
	 * internim udajum objednavky
	 * 
	 * - verifikace podpisu
	 * 
	 *
	 * @param mixed $payment_status - vysledek volani paymentStatus
	 * @param string $session_state - ocekavany stav paymentSession (WAITING, PAYMENT_DONE)
	 * @param long $goId - identifikace eshopu
	 * @param string $variableSymbol - identifikace akt. objednavky
	 * @param long $priceInCents - cena objednavky v centech
	 * @param string $productName - nazev zbozi
	 * @param string $secret - klic urceny k podepisovani komunikace 
	 * 
	 * @return true
	 * @return false
	 */
	
	function checkPaymentStatus(
  				$payment_status,
  	 			$session_state,
  	 			$goId,
  	 			$variableSymbol,  	 			
  	 			$priceInCents,
  	 			$productName,
  	 			$secret	 			
  	 			){

		$valid = true;
		
		$valid = GopayHelper::checkPaymentResultCommon(
  				$payment_status,
  	 			$session_state,
  	 			$goId,
  	 			$variableSymbol,  	 			
  	 			$priceInCents,
  	 			$productName,
  	 			$secret	 					
		);
		

		if( $valid){
		
			$hashedSignature=GopayHelper::hash(
					GopayHelper::concatPaymentStatus(
						$payment_status->eshopGoId,
						$payment_status->productName,						
						$payment_status->totalPrice,
						$payment_status->variableSymbol,
						$payment_status->result,
						$payment_status->sessionState,
						$payment_status->paymentChannel,
						$secret)
			);			

			$decryptedHash = GopayHelper::decrypt($payment_status->encryptedSignature,$secret);

			if($decryptedHash != $hashedSignature){
				$valid = false;
//				echo "PS invalid signature <br>";
			}

		}else{
			$valid = false;
//			echo "none payment status <br>";
		}

		return $valid;
	}
	
  	private function checkPaymentResultCommon(
  				$payment_result,
  	 			$session_state,
  	 			$goId,
  	 			$variableSymbol,  	 			
  	 			$priceInCents,
  	 			$productName,
  	 			$secret	 			
  	 			){

		$valid = true;

		if( $payment_result){

			if( $payment_result->result != 'CALL_COMPLETED'){
				$valid = false;
//				echo "PS invalid call state state<br>";
			}

			if( $payment_result->sessionState != $session_state){
				$valid = false;
//				echo "PS invalid session state<br>";			
			}

			if($payment_result->variableSymbol != $variableSymbol){
				$valid = false;
//				echo "PS invalid VS <br>";
			}

			if($payment_result->productName != $productName){
				$valid = false;
//				echo "PS invalid PN <br>";
			}

			if($payment_result->eshopGoId != $goId){
				$valid = false;
//				echo "PS invalid EID<br>";
			}

			if($payment_result->totalPrice != $priceInCents){
				$valid = false;
//				echo "PS invalid price<br>";
			}

		}else{
			$valid = false;
//			echo "none payment status <br>";
		}

		return $valid;
	}
	
	
	/**
	 * Kontrola parametru predavanych ve zpetnem volani
	 * po potvrzeni/zruseni platby
	 * 
	 * - verifikace podpisu
	 * 
	 *
	 * @param long $returnedGoId - goId vracene v redirectu
	 * @param long $returnedPaymentSessionId - paymentSessionId vracene v redirectu
	 * @param string $returnedVariableSymbol - variableSymbol vracene v redirectu
	 * @param string $returnedEncryptedSignature - encryptedSignature vracene v redirectu 
	 * @param float $goId - identifikace eshopu
	 * @param string $variableSymbol - oznaceni objednavky
	 * @param string $secret - klic urceny k podepisovani komunikace
	 * 
	 * @return true
	 * @return false
	 */
  	function checkPaymentIdentity(
  				$returnedGoId,
  				$returnedPaymentSessionId,  				
  				$returnedVariableSymbol,
  				$returnedEncryptedSignature,
  	 			$goId,
  	 			$variableSymbol,  	 			
  	 			$secret  	 			
  	 			){

		$valid = true;
		if($returnedVariableSymbol != $variableSymbol){
			$valid = false;
			echo "PI invalid VS <br>";
		}

		if($returnedGoId != $goId){
			$valid = false;
			echo "PI invalid EID<br>";
		}

		$hashedSignature=GopayHelper::hash(
				GopayHelper::concatPaymentIdentity(
					$returnedGoId,
					$returnedPaymentSessionId,						
					$returnedVariableSymbol,
					$secret)
			);
		$decryptedHash = GopayHelper::decrypt($returnedEncryptedSignature, $secret);

		if($decryptedHash != $hashedSignature){
			$valid = false;
			echo "PI invalid signature <br>";
		}

		return $valid;
	}

}
?>